@@ -369,7 +369,7 @@ end subroutine find_variables_associates
     routine = Subroutine.from_source(fcode, frontend=frontend)
 
     variables = FindVariables(unique=False).visit(routine.body)
-    assert len(variables) == 29
+    assert len(variables) == 27 if frontend == OMNI else 28
     assert len([v for v in variables if v.name == 'v']) == 1
     assert len([v for v in variables if v.name == 'm']) == 2
 
