@@ -5,13 +5,13 @@
 # granted to it by virtue of its status as an intergovernmental organisation
 # nor does it submit to any jurisdiction.
 
+from collections import defaultdict
 from loki.batch import Transformation
-from loki.expression import (
-    symbols as sym, FindVariables, is_dimension_constant
-)
+from loki.expression import symbols as sym, is_dimension_constant
 from loki.ir import (
-    nodes as ir, FindNodes, pragmas_attached, is_loki_pragma,
-    get_pragma_parameters, Transformer
+    nodes as ir, FindNodes, FindVariables, Transformer,
+    pragmas_attached, is_loki_pragma, get_pragma_parameters,
+    pragma_regions_attached
 )
 from loki.logging import info
 from loki.tools import as_tuple, flatten
@@ -194,11 +194,59 @@ class SCCAnnotateTransformation(Transformation):
             # Mark all non-parallel loops as `!$acc loop seq`
             self.annotate_sequential_loops(routine)
 
-            with pragmas_attached(routine, ir.Loop, attach_pragma_post=True):
-                driver_loops = find_driver_loops(routine=routine, targets=targets)
-                for loop in driver_loops:
-                    self.annotate_driver_loop(loop)
+            with pragma_regions_attached(routine):
+                with pragmas_attached(routine, ir.Loop, attach_pragma_post=True):
+                    # Find variables with existing OpenACC data declarations
+                    acc_vars = self.find_acc_vars(routine, targets)
 
+                    driver_loops = find_driver_loops(section=routine.body, targets=targets)
+                    for loop in driver_loops:
+                        self.annotate_driver_loop(loop, acc_vars.get(loop, []))
+
+    def find_acc_vars(self, routine, targets):
+        """
+        Find variables already specified in acc data clauses.
+
+        Parameters
+        ----------
+        routine : :any:`Subroutine`
+            Subroutine to apply this transformation to.
+        targets : list or string
+            List of subroutines that are to be considered as part of
+            the transformation call tree.
+        """
+
+        acc_vars = defaultdict(list)
+
+        for region in FindNodes(ir.PragmaRegion).visit(routine.body):
+            if region.pragma.keyword.lower() == 'acc':
+                if (parameters := get_pragma_parameters(region.pragma, starts_with='data', only_loki_pragmas=False)):
+
+                    driver_loops = find_driver_loops(section=region.body, targets=targets)
+                    if not driver_loops:
+                        continue
+
+                    # When a key is given multiple times, get_pragma_parameters returns a list
+                    # We merge them here into single entries to make our life easier below
+                    parameters = {key: ', '.join(as_tuple(value)) for key, value in parameters.items()}
+
+                    if (default := parameters.get('default', None)):
+                        if not 'none' in [p.strip().lower() for p in default.split(',')]:
+                            for loop in driver_loops:
+
+                                _vars = [var.name.lower() for var in FindVariables(unique=True).visit(loop)]
+                                acc_vars[loop] += _vars
+                    else:
+                        _vars = [
+                            p.strip().lower()
+                            for category in ('present', 'copy', 'copyin', 'copyout', 'deviceptr')
+                            for p in parameters.get(category, '').split(',')
+                        ]
+
+                        for loop in driver_loops:
+                            acc_vars[loop] += _vars
+
+        return acc_vars
 
     @classmethod
     def device_alloc_column_locals(cls, routine, column_locals):
@@ -221,14 +269,16 @@ class SCCAnnotateTransformation(Transformation):
             routine.body.prepend((ir.Comment(''), pragma, ir.Comment('')))
             routine.body.append((ir.Comment(''), pragma_post, ir.Comment('')))
 
-    def annotate_driver_loop(self, loop):
+    def annotate_driver_loop(self, loop, acc_vars):
         """
         Annotate driver block loop with ``'openacc'`` pragmas.
 
         Parameters
         ----------
         loop : :any:`Loop`
-            Driver :any:`Loop` to wrap in ``'opencc'`` pragmas.
+            Driver :any:`Loop` to wrap in ``'openacc'`` pragmas.
+        acc_vars : list
+            Variables already declared in ``'openacc'`` data directives.
         """
 
         # Mark driver loop as "gang parallel".
@@ -241,7 +291,7 @@ class SCCAnnotateTransformation(Transformation):
             # Filter out arrays that are explicitly allocated with block dimension
             sizes = self.block_dim.size_expressions
             arrays = [v for v in arrays if not any(d in sizes for d in as_tuple(v.shape))]
-            private_arrays = ', '.join(set(v.name for v in arrays))
+            private_arrays = ', '.join(set(v.name for v in arrays if not v.name_parts[0].lower() in acc_vars))
             private_clause = '' if not private_arrays else f' private({private_arrays})'
 
             for pragma in as_tuple(loop.pragma):
