@@ -35,7 +35,7 @@ def fixture_local_loki_bundle(here):
 def fixture_bundle_create(here, local_loki_bundle):
     """Inject ourselves into the ECWAM bundle"""
     env = os.environ.copy()
-    env['ECWAM_CONCEPT_BUNDLE_LOKI_DIR'] = local_loki_bundle
+    env['ECWAM_BUNDLE_LOKI_DIR'] = local_loki_bundle
 
     # Run ecbundle to fetch dependencies
     execute(
